﻿using LawFirmBusinessLogic.MailWorker;
using LawFirmContracts.BindingModels;
using LawFirmContracts.BusinessLogicsContracts;
using LawFirmContracts.SearchModels;
using LawFirmContracts.StoragesContracts;
using LawFirmContracts.ViewModels;
using LawFirmDataModels.Enums;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace LawFirmBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
		private readonly IClientStorage _clientStorage;
		private readonly AbstractMailWorker _mailLogic;
		static readonly object locker = new object();

		public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage, IClientStorage clientStorage, AbstractMailWorker worker)
        {
            _logger = logger;
            _orderStorage = orderStorage;
			_clientStorage = clientStorage;
			_mailLogic = worker;
		}

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() :
           _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }
        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен) return false;
            model.Status = OrderStatus.Принят;
			var order = _orderStorage.Insert(model);
			if (order == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			var clientView = _clientStorage.GetElement(new() { Id = order!.ClientId });

			SendEmail(clientView, order);
			return true;
        }

        public bool ChangeStatus(OrderBindingModel model, OrderStatus status)
        {
            CheckModel(model, false);
            var element = _orderStorage.GetElement(new OrderSearchModel { Id = model.Id });
            if (element == null)
            {
                _logger.LogWarning("Read operation failed");
                return false;
            }
            if (element.Status != status - 1)
            {
                _logger.LogWarning("Status change operation failed");
                throw new InvalidOperationException("Текущий статус заказа не может быть переведен в выбранный");
            }
            model.Status = status;
            if (model.Status == OrderStatus.Выдан) model.DateImplement = DateTime.Now;
            if (element.ImplementerId.HasValue) model.ImplementerId = element.ImplementerId;
            element = _orderStorage.Update(model);
            if(element == null)
            {
				_logger.LogWarning("Update operation failed");
				return false;
            }
			var clientView = _clientStorage.GetElement(new() { Id = element.ClientId });

			SendEmail(clientView, element);
			return true;
        }

        public bool TakeOrderInWork(OrderBindingModel model)
        {
            lock (locker)
            {
                return ChangeStatus(model, OrderStatus.Выполняется);
            }
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Готов);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return ChangeStatus(model, OrderStatus.Выдан);
        }

        private void CheckModel(OrderBindingModel model, bool withParams =
true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Цена заказа должна быть больше 0", nameof(model.Sum));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество элементов в заказе должно быть больше 0", nameof(model.Count));
            }
            _logger.LogInformation("Order. Sum:{ Cost}. Id: { Id}", model.Sum, model.Id);
        }
		public void SendEmail(ClientViewModel clientModel, OrderViewModel orderModel)
		{
			if (clientModel == null && orderModel == null)
			{
				return;
			}

			MailSendInfoBindingModel mailModel;

			if (orderModel.Status == OrderStatus.Принят)
			{
				mailModel = new MailSendInfoBindingModel
				{
					MailAddress = clientModel.Email,
					Subject = $"Order №{orderModel.Id}",
					Text = $"Your order №{orderModel.Id} by {orderModel.DateCreate} on {orderModel.Sum} was accepted!"
				};
			}
			else
			{
				mailModel = new MailSendInfoBindingModel
				{
					MailAddress = clientModel.Email,
					Subject = $"Order №{orderModel.Id}",
					Text = $"Order №{orderModel.Id} status was changed to {orderModel.Status}"
				};
			}


			_mailLogic.MailSendAsync(mailModel);
		}
	}
}
